@@ -2,7 +2,6 @@ import requests
 import pandas as pd
 import numpy as np
 import datetime as dt
-from dateutil.relativedelta import relativedelta
 
 TVL_BASE_URL = "https://api.llama.fi"
 COINS_BASE_URL = "https://coins.llama.fi"
@@ -256,10 +255,12 @@ class DefiLlama:
         df = df.set_index('timestamp')
         return df 
 
-    def get_tokens_hist_prices(self, token_addrs_n_chains, start, end, type='close'):
-        """Get historical prices of tokens by contract address.
+    def get_tokens_hist_prices(self, token_addrs_n_chains, start, end, freq='hourly'):
+        """Get historical hourly or daily prices of tokens by contract address.
         Uses get_tokens_hist_snapshot_prices() to download iteratively since
-        DeFiLlama currently doesn't offer an API for bulk download. 
+        DeFiLlama currently doesn't offer an API for bulk download. If you want 
+        to get daily open or close prices, use get_daily_open_close() instead 
+        since it's faster.
         
         Parameters
         ----------
@@ -273,8 +274,8 @@ class DefiLlama:
             Start date, for example, '2021-01-01'
         end : string
             End date, for example, '2022-01-01'
-        type : string
-            Price type, either 'close' (default) or 'open'. Does NOT support 
+        freq : string
+            Data granularity, 'hourly' (default) or 'daily'. Does NOT support 
             other values at the moment.
 
         Returns 
@@ -283,14 +284,85 @@ class DefiLlama:
         """
         start = dt.datetime.strptime(start, '%Y-%m-%d')
         end   = dt.datetime.strptime(end, '%Y-%m-%d')
+        timestamps = pd.date_range(start, end, freq='60min')
+        timestamps = timestamps[timestamps < dt.datetime.now()-pd.Timedelta(hours=4)] # give 4 hours buffer to ensure DeFiLlama data are available
+         
+        # download historical hourly data
+        df = pd.concat(self.get_tokens_hist_snapshot_prices(token_addrs_n_chains, dttm) for dttm in timestamps)
+
+        # clean data so that the resulting frame has 
+        #   - each row is a datetime
+        #   - each column is a token
+        #   - each value is a price (open or close)
+        df = df.reset_index()
+        df['datetime'] = [elt.round(freq='H') for elt in df.timestamp]
+        # df[['timestamp', 'datetime']].head(10)
+        # there can be duplicates in the col `datetime`, so take their avg price
+        df = df.groupby(['datetime', 'symbol'])['price'].mean() 
+        df = df.reset_index().pivot(index='datetime', columns='symbol', values='price')
+        df.columns.name = None
+        df.index = pd.to_datetime(df.index)
+        
+        # derive daily prices if user requests them instead of hourly data
+        if freq == 'daily':
+            symbols = df.columns
+            # calculate daily prices using hourly data
+            daily_open = df.asfreq('1d')
+            daily_low = df.resample('D').min()
+            daily_high = df.resample('D').max()
+            daily_close = daily_open.shift(-1)
+            daily_med = df.resample('D').median()
+            daily_avg = df.resample('D').mean()
+            daily_std = df.resample('D').std()
+            # assign header 
+            daily_open.columns = pd.MultiIndex.from_tuples([('open', symbol) for symbol in symbols])
+            daily_low.columns = pd.MultiIndex.from_tuples([('low', symbol) for symbol in symbols])
+            daily_high.columns = pd.MultiIndex.from_tuples([('high', symbol) for symbol in symbols])
+            daily_close.columns = pd.MultiIndex.from_tuples([('close', symbol) for symbol in symbols])
+            daily_med.columns = pd.MultiIndex.from_tuples([('median', symbol) for symbol in symbols])
+            daily_avg.columns = pd.MultiIndex.from_tuples([('mean', symbol) for symbol in symbols])
+            daily_std.columns = pd.MultiIndex.from_tuples([('std', symbol) for symbol in symbols])
+            # join together
+            df = pd.concat([daily_open, daily_low, daily_high, daily_close, daily_med, daily_avg, daily_std], axis=1)
+        return df
+
+    def get_daily_open_close(self, token_addrs_n_chains, start, end, kind='close'):
+        """Get historical daily open and close prices of tokens by contract address.
+        Uses get_tokens_hist_snapshot_prices() to download iteratively since
+        DeFiLlama currently doesn't offer an API for bulk download. 
+        
+        Parameters
+        ----------
+        token_addrs_n_chains : dictionary
+            Each key is a token address; each value is a chain where the token 
+            address resides. If getting price from coingecko, use token name as 
+            key and 'coingecko' as value. For example, 
+            {'0xdF574c24545E5FfEcb9a659c229253D4111d87e1':'ethereum',
+                'ethereum':'coingecko'}
+        start : string
+            Start date, for example, '2021-01-01'
+        end : string
+            End date, for example, '2022-01-01'
+        kind : string
+            Either 'close' (default, at 23:59:59) or 'open' (at 00:00:00). Does NOT 
+            support other values at the moment.
+
+        Returns 
+        -------
+        data frame
+        """
+        start = dt.datetime.strptime(start, '%Y-%m-%d')
+        end   = dt.datetime.strptime(end, '%Y-%m-%d')
+        if (end.date() == dt.date.today()) and (kind == 'close'): 
+            end -= pd.Timedelta(days=1)
         dates = pd.date_range(start, end)
 
-        if type == 'close':
+        if kind == 'close':
             dttms = [date.replace(hour=23, minute=59, second=59) for date in dates] 
-        elif type == 'open':
+        elif kind == 'open':
             dttms = [date.replace(hour=0, minute=0, second=0) for date in dates] 
         else: 
-            raise Exception("Only 'open' or 'close' are supported for `type`.")
+            raise Exception("Only 'open' or 'close' are supported.")
 
         # download historical snapshots one by one    
         df = pd.concat(self.get_tokens_hist_snapshot_prices(token_addrs_n_chains, dttm) for dttm in dttms)
@@ -300,17 +372,18 @@ class DefiLlama:
         #   - each column is a token
         #   - each value is a price (open or close)
         df = df.reset_index()
-        if type == 'close':
+        if kind == 'close':
             df['date'] = np.where(df.timestamp.dt.hour == 0, 
-                                  df.timestamp.dt.date - relativedelta(days=1), 
+                                  df.timestamp.dt.date - pd.Timedelta(days=1), 
                                   df.timestamp.dt.date)
-        if type == 'open':
+        if kind == 'open':
             df['date'] = np.where(df.timestamp.dt.hour == 0, 
                                   df.timestamp.dt.date, 
-                                  df.timestamp.dt.date + relativedelta(days=1))
+                                  df.timestamp.dt.date + pd.Timedelta(days=1))
         df = df.groupby(['date', 'symbol'])['price'].mean()
         df = df.reset_index().pivot(index='date', columns='symbol', values='price')
         df.columns.name = None
+        df.index = pd.to_datetime(df.index)
         return df
 
     def get_closest_block(self, chain, timestamp):
